<?php

namespace Drupal\commerce_webpay_by\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Class WebpayByRedirectForm
 */
class WebpayByRedirectForm extends BasePaymentOffsiteForm {

  const PRECISION = 2;

  const DEC_POINT = '.';

  const THOUSANDS_SEP = '';

  const ROUND_MODE = PHP_ROUND_HALF_UP;

  /**
   * Gateway plugin.
   *
   * @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface
   */
  private $paymentGatewayPlugin;


  /**
   * Getting plugin's configuration.
   *
   * @param string $configuration
   *   Configuration name.
   *
   * @return mixed
   *   Configuration value.
   */
  private function getConfiguration($configuration) {
    return $this->paymentGatewayPlugin->getConfiguration()[$configuration] ?? NULL;
  }


  /**
   * Get payment gateway default settings.
   *
   * @param null $name
   *
   * @return array|\Drupal\Core\Config\ImmutableConfig|mixed|null
   */
  private function getDefaultSettings($name = NULL) {
    $config = \Drupal::config('commerce_webpay_by.settings');
    return $name ? $config->get("commerce_webpay_by.{$name}") ?? NULL : $config;
  }


  /**
   * @param array $data
   *
   * @return string
   */
  private function signature(array $data) {
    $signature = '';

    /**
     * @var string $wsb_seed
     * @var string $wsb_storeid
     * @var string $wsb_order_num
     * @var string $wsb_test
     * @var string $wsb_currency_id
     * @var string $wsb_total
     * @var string $secret_key
     */
    extract($data);

    $signature .= $wsb_seed;
    $signature .= $wsb_storeid;
    $signature .= $wsb_order_num;
    $signature .= $wsb_test;
    $signature .= $wsb_currency_id;
    $signature .= $wsb_total;
    $signature .= $secret_key;

    return sha1($signature);
  }


  /**
   * Round price.
   *
   * @param float $number
   * @param int   $precision
   * @param int   $mode
   *
   * @return float
   */
  private function round(float $number, $precision = self::PRECISION, $mode = self::ROUND_MODE) {
    return (float) number_format(round($number, $precision, $mode), $precision, self::DEC_POINT, self::THOUSANDS_SEP);
  }


  /**
   * Create URL.
   *
   * @param string $route
   *
   * @return \Drupal\Core\GeneratedUrl|string
   */
  public function url(string $route) {
    return Url::fromRoute($route, [], ['absolute' => TRUE])->toString();
  }


  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    $this->paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();

    $order = $payment->getOrder();
    $sandbox = $this->getConfiguration('mode');
    $redirect_url = $this->getDefaultSettings("{$sandbox}_uri");
    $total_price = 0;
    $shipping_price = 0;
    $data = [];

    $data['*scart'] = NULL;
    $data['wsb_storeid'] = $this->getConfiguration('wsb_storeid');
    $data['wsb_order_num'] = $payment->getOrderId();
    $data['wsb_currency_id'] = $this->getConfiguration('wsb_currency_id');
    $data['wsb_version'] = $this->getDefaultSettings('wsb_version');
    $data['wsb_seed'] = time();
    $data['wsb_notify_url'] = $this->url('commerce_webpay_by.webpay_by_redirect_controller_notify');

    if ($wsb_language_id = $this->getConfiguration('wsb_language_id')) {
      $data['wsb_language_id'] = $wsb_language_id;
    }
    if ($wsb_store = $this->getConfiguration('wsb_store')) {
      $data['wsb_store'] = \Drupal::token()->replace($wsb_store, ['commerce_order' => $order]);
    }
    if ($wsb_return_url = $this->getConfiguration('wsb_return_url')) {
      $data['wsb_return_url'] = \Drupal::token()->replace($wsb_return_url, ['commerce_order' => $order]);
    }
    if ($wsb_cancel_return_url = $this->getConfiguration('wsb_cancel_return_url')) {
      $data['wsb_cancel_return_url'] = \Drupal::token()->replace($wsb_cancel_return_url, ['commerce_order' => $order]);
    }

    /** @var \Drupal\user\Entity\User $user */
    if ($user = \Drupal::currentUser()) {
      $data['wsb_email'] = $user->getEmail();
    }

    $i = 0;
    /** @var \Drupal\commerce_order\Entity\OrderItem $item */
    foreach ($order->getItems() as $item) {
      $price_per_item = $this->round($item->getUnitPrice()->getNumber());
      $quantity = round($item->getQuantity(), 0);
      $data["wsb_invoice_item_name[{$i}]"] = $item->getTitle();
      $data["wsb_invoice_item_quantity[{$i}]"] = $quantity;
      $data["wsb_invoice_item_price[{$i}]"] = $price_per_item;
      $total_price += $price_per_item * $quantity;
      $i++;
    }

    /** @var \Drupal\commerce_order\Adjustment $item */
    foreach ($order->getAdjustments() as $item) {
      if ($item->getType() === 'shipping') {
        $shipping_price += $this->round($item->getAmount()->getNumber());
      }
    }

    $discount = $this->round($order->getSubtotalPrice()->getNumber() - $order->getTotalPrice()->getNumber());

    $data['wsb_shipping_price'] = $this->round($shipping_price);
    $data['wsb_discount_price'] = $discount + $data['wsb_shipping_price'];
    $data['wsb_total'] = $this->round($total_price + $data['wsb_shipping_price'] - $data['wsb_discount_price']);
    $data['wsb_test'] = $sandbox;
    $data['secret_key'] = $this->getConfiguration('secret_key');
    $data['wsb_signature'] = $this->signature($data);

    return $this->buildRedirectForm($form, $form_state, $redirect_url, $data, self::REDIRECT_POST);
  }
}
